from time import sleep, time
import os

from pymk.tests.base import PymkTestCase
from pymk.extra import touch


class TouchTest(PymkTestCase):
    test_file = 'testme.file'

    def test_new_file(self):
        actual_time = int(time())
        touch(self.test_file)
        file_time = int(os.path.getmtime(self.test_file))
        self.assertTrue(os.path.exists(self.test_file))
        self.assertEqual(actual_time, file_time)

    def test_change_time(self):
        touch(self.test_file)
        first_file_time = os.path.getmtime(self.test_file)

        sleep(0.01)
        touch(self.test_file)
        second_file_time = os.path.getmtime(self.test_file)

        self.assertNotEqual(first_file_time, second_file_time)



from mock import patch

from pymk.tests.base import PymkTestCase
from pymk.extra.template import mktemplate, Template


class TemplateTest(PymkTestCase):

    def _init_patchers(self):
        super(TemplateTest, self)._init_patchers()
        self.patchers['cache'] = patch.dict(Template.cache, {})
        self.patchers['PackageLoader'] = patch(
            'pymk.extra.template.PackageLoader')
        self.patchers['Environment'] = patch('pymk.extra.template.Environment')

    def test_get_env(self):
        env = Template().get_env()

        self.assertEqual(env, Template().cache['env'])
        self.mocks['PackageLoader'].assert_called_once_with(
            'pymktemplates', '.')
        loader = self.mocks['PackageLoader'].return_value
        self.mocks['Environment'].assert_called_once_with(loader=loader)
        self.assertEqual(env, self.mocks['Environment'].return_value)

    def test_get_env_with_env(self):
        Template.cache['env'] = 'something'
        env = Template().get_env()

        self.assertEqual('something', env)
        self.assertEqual(0, self.mocks['PackageLoader'].call_count)
        self.assertEqual(0, self.mocks['Environment'].call_count)

    def test_make(self):
        with patch.object(Template, 'get_env') as get_env:
            env = get_env.return_value

            Template().make('template_path', 'output_file', {'data': 1})

            env.get_template.assert_called_once_with('template_path')
            template = env.get_template.return_value
            template.stream.assert_called_once_with(**{'data': 1})
            stream = template.stream.return_value
            stream.dump.assert_called_once_with('output_file')

    def test_mktemplate(self):
        with patch('pymk.extra.template.Template') as Template:
            mktemplate('template_path', 'output_file', 'data')

            Template.assert_called_once_with()
            template = Template.return_value
            template.make.assert_called_once_with(
                'template_path', 'output_file', 'data')



import os
import shutil
import urllib2
import urlparse
from zipfile import ZipFile


# this code is from
# http://stackoverflow.com/questions/862173/how-to-download-a-file-using-python-in-a-smarter-way
def download(url, fileName=None):
    def getFileName(url, openUrl):
        if 'Content-Disposition' in openUrl.info():
            # If the response has Content-Disposition, try to get filename from
            # it
            cd = dict(map(
                lambda x: x.strip().split(
                    '=') if '=' in x else (x.strip(), ''),
                openUrl.info()['Content-Disposition'].split(';')))
            if 'filename' in cd:
                filename = cd['filename'].strip("\"'")
                if filename:
                    return filename
        # if no filename was found above, parse it out of the final URL.
        return os.path.basename(urlparse.urlsplit(openUrl.url)[2])

    r = urllib2.urlopen(urllib2.Request(url))
    try:
        fileName = fileName or getFileName(url, r)
        with open(fileName, 'wb') as f:
            shutil.copyfileobj(r, f)
    finally:
        r.close()

# end of external code


def extract_egg(source_path, destination_path):
    zipfile = ZipFile(source_path)
    zipfile.extractall(destination_path)
    zipfile.close()
    os.unlink(source_path)



class PymkError(Exception):

    """Base exception for pymk. Do nothing special.
    """


class CouldNotCreateFile(PymkError):

    def __init__(self, filename):
        self.filename = filename

    def __str__(self):
        return 'Error: Could not create file %s' % (self.filename)


class TaskAlreadyExists(PymkError):

    def __init__(self, task_name):
        self.task_name = task_name

    def __str__(self):
        return 'Error: Task name already exists "%s".' % (self.task_name)


class RecipeAlreadyExists(PymkError):

    def __init__(self, recipe_name):
        self.recipe_name = recipe_name

    def __str__(self):
        return 'Error: Recipe name already exists "%s".' % (self.recipe_name)


class NoMkfileFound(PymkError):
    pass


class CommandError(PymkError):

    def __init__(self, number, text):
        self.number = number
        self.text = text

    def __str__(self):
        return 'Error: Command error (%d): %s' % (self.number, self.text)


class CommandAborted(PymkError):
    pass


class BadTaskPath(PymkError):

    def __init__(self, taskpath):
        self.taskpath = taskpath

    def __str__(self):
        return 'Error: Bad task path: %s' % (self.taskpath)


class WrongArgumentValue(PymkError):

    def __init__(self, description):
        self.description = description


class TaskMustHaveOutputFile(PymkError):

    def __init__(self, name):
        self.name = name

    def __str__(self):
        return 'Error: Taks must have output_file setted: %s' % (self.name)


class NoDependencysInAClass(PymkError):

    def __init__(self, cls):
        self.cls = cls

    def __str__(self):
        return 'Error: Class %s has no "dependencys" attribute or it has wrong name.' % (self.cls.__name__)


class NotADependencyError(PymkError):

    def __init__(self, dependency, task):
        self.dependency = dependency
        self.task = task

    def __str__(self):
        return 'Error: Object "%s" of a task "%s" is not a dependency!' % (str(self.dependency.__name__), self.task.__name__)


class WrongPymkVersion(PymkError):

    def __init__(self, pymk_version, mkfile_version):
        self.pymk_version = pymk_version
        self.mkfile_version = mkfile_version

    def __str__(self):
        return 'Error: mkfile need greater version (%s) of pymk (%s). Please update.' % (self.mkfile_version, self.pymk_version)



import unittest
from pymk import error


class ErrorsTests(unittest.TestCase):

    def test_CouldNotCreateFile(self):
        er = error.CouldNotCreateFile('filename')
        self.assertEqual(str, type(str(er)))

    def test_TaskAlreadyExists(self):
        er = error.TaskAlreadyExists('task name')
        self.assertEqual(str, type(str(er)))

    def test_CommandError(self):
        er = error.CommandError(1, 'task name')
        self.assertEqual(str, type(str(er)))

    def test_BadTaskName(self):
        er = error.BadTaskPath('task name')
        self.assertEqual(str, type(str(er)))

    def test_WrongArgumentValue(self):
        er = error.WrongArgumentValue('description')
        self.assertEqual(str, type(str(er)))

    def test_TaskMustHaveOutputFile(self):
        er = error.TaskMustHaveOutputFile('name')
        self.assertEqual(str, type(str(er)))

    def test_NoDependencysInAClass(self):
        er = error.NoDependencysInAClass(error.NoDependencysInAClass)
        self.assertEqual(str, type(str(er)))

    def test_NotADependencyError(self):
        er = error.NotADependencyError(
            error.NoDependencysInAClass, error.NoDependencysInAClass)
        self.assertEqual(str, type(str(er)))

    def test_RecipeAlreadyExists(self):
        er = error.RecipeAlreadyExists('name')
        self.assertEqual(str, type(str(er)))

    def test_WrongPymkVersion(self):
        er = error.WrongPymkVersion('pymk_version', 'mkfile_version')
        self.assertEqual(str, type(str(er)))



from time import sleep

import StringIO
from mock import patch, MagicMock

import pymk.error as Perror
from pymk.dependency import FileChanged, FileDoesNotExists, AlwaysRebuild, InnerFileExists, InnerFileChanged, Dependency, FileDependency
from pymk.task import Task
from pymk.tests.base import PymkTestCase
from pymk.dependency import InnerLink


class FileDoesNotExistsDependencyTest(PymkTestCase):

    def test_make(self):
        self._template('one_task_dependency_1', 'mkfile.py')
        self._import_mkfile()

        self._args.task.append('/task_2')
        self._pymk_runtask(['task_2'])
        self._pymk_runtask(['task_2', 'task_2'])
        self._pymk_runtask(['task_2', 'task_2', 'task_2'])

    def test_uptodate(self):
        self._template('one_task_dependency_1', 'mkfile.py')
        self._template('one_task_dependency_1', 'test.txt')
        self._import_mkfile()

        self._add_task('/task_2')
        self._pymk_runtask([])
        self._pymk_runtask([])

    def test_graph(self):
        dep = FileDoesNotExists('')

        self.assertEqual(str, type(dep.extra()))
        self.assertEqual(str, type(dep.get_graph_name()))
        self.assertEqual(str, type(dep.get_graph_details()))

        dep.runned = True

        self.assertEqual(str, type(dep.extra()))
        self.assertEqual(str, type(dep.get_graph_name()))
        self.assertEqual(str, type(dep.get_graph_details()))

        output = StringIO.StringIO()
        dep.write_graph_detailed(output)
        self.assertNotEqual(0, len(output.getvalue().strip()))

    def test_check_force(self):
        dep = FileDoesNotExists('filename')
        self.assertTrue(dep.do_test(None, True))

    def test_two_files(self):
        file1 = 'test.txt'
        file2 = 'test2.txt'
        taskname = 'task_22'
        self._template('one_task_dependency_5', 'mkfile.py')
        self._import_mkfile()

        self._add_task('/' + taskname)

        self._pymk_runtask([taskname])
        self._pymk_runtask([taskname, taskname])

        self.touch(file1, None)
        self._pymk_runtask([taskname, taskname, taskname])

        self.touch(file2, None)
        self._pymk_runtask([taskname, taskname, taskname])

    def test_extra(self):
        depedency = Dependency()
        self.assertEqual('', depedency.extra())

    def test_get_graph_details(self):
        depedency = Dependency()
        self.assertEqual('', depedency.get_graph_details())

    def test_get_shape_color(self):
        depedency = Dependency()
        depedency.runned = False
        self.assertEqual('white', depedency._get_shape_color())

    def test_get_shape_color_after_run(self):
        depedency = Dependency()
        depedency.runned = True
        self.assertEqual('darkgreen', depedency._get_shape_color())


class FileDependencyTest(PymkTestCase):

    filename = 'something.txt'

    def setUp(self):
        super(FileDependencyTest, self).setUp()
        self.dep = FileDependency(self.filename)

    def test_init(self):
        self.assertEqual([self.filename, ], self.dep.filenames)

    def test_name(self):
        self.assertEqual('something.txt', self.dep.name)

    def test_name_many_files(self):
        self.dep = FileDependency([self.filename, 'filename2.txt'])
        self.assertEqual('Many files', self.dep.name)

    @patch('pymk.dependency.os.path.getmtime')
    def test_compare_mtime(self, getmtime):
        getmtime.return_value = 1
        self.assertFalse(self.dep.compare_mtime('1', '2'))
        self.assertEqual(2, getmtime.call_count)
        getmtime.assert_called_with('2')
        self.assertEqual(('1',), getmtime.mock_calls[0][1])
        self.assertEqual(('2',), getmtime.mock_calls[1][1])

    @patch('pymk.dependency.os.path.getmtime')
    def test_compare_mtime_true(self, getmtime):
        def side_effect_1(*args, **kwargs):
            return 1

        def side_effect_2(*args, **kwargs):
            getmtime.side_effect = side_effect_1
            return 2
        getmtime.side_effect = side_effect_2

        self.assertTrue(self.dep.compare_mtime('1', '2'))
        self.assertEqual(2, getmtime.call_count)
        getmtime.assert_called_with('2')
        self.assertEqual(('1',), getmtime.mock_calls[0][1])
        self.assertEqual(('2',), getmtime.mock_calls[1][1])


class FileChangedDependencyTest(PymkTestCase):

    def test_check_dependent_file_no_file(self):
        dep = FileChanged('something', MagicMock())
        with patch.object(dep, 'compare_mtime') as compare_mtime:
            with patch.object(dep, 'make_dependent_file') as make_dependent_file:
                compare_mtime.side_effect = OSError()

                dep.check_dependent_file(MagicMock())
                make_dependent_file.assert_called_once_with()

    def test_check_dependent_file(self):
        dep = FileChanged('something', MagicMock())
        with patch.object(dep, 'compare_mtime') as compare_mtime:
            result = dep.check_dependent_file(MagicMock())
            compare_mtime.called_once_with('something')
            self.assertEqual(compare_mtime.return_value.__ror__(), result)

    def test_FileChanged_dependency_make_fail(self):
        self._template('one_task_dependency_2', 'mkfile.py')
        self._import_mkfile()

        self._add_task('/task_3')

        self.assertRaises(Perror.TaskMustHaveOutputFile, self._pymk)
        self._check_output_file([])

    def test_FileChanged_dependency_make_fail_2(self):
        self._template('one_task_dependency_2', 'mkfile.py')
        self.touch('test.txt', None)
        self._import_mkfile()

        self._add_task('/task_3')

        self.assertRaises(Perror.TaskMustHaveOutputFile, self._pymk)
        self._check_output_file([])

    def test_FileChanged_dependency_make_faile_3(self):
        self._template('one_task_dependency_3', 'mkfile.py')
        self._import_mkfile()

        self._add_task('/task_4')

        self.assertRaises(Perror.CouldNotCreateFile, self._pymk)
        self._check_output_file([])

    def test_FileChanged_dependency_make_once(self):
        self._template('one_task_dependency_3', 'mkfile.py')
        self._import_mkfile()

        self._add_task('/task_4')

        self.touch('test.txt', None)
        self._pymk_runtask(['task_4'])
        self._pymk_runtask(['task_4'])

    def test_FileChanged_dependency_make_twice(self):
        self._template('one_task_dependency_3', 'mkfile.py')
        self._import_mkfile()

        self._add_task('/task_4')

        self.touch('test.txt', None)
        self._pymk_runtask(['task_4'])

        self.touch('test.txt')
        self._pymk_runtask(['task_4', 'task_4'])

        sleep(0.001)
        self._pymk_runtask(['task_4', 'task_4'])

    def test_FileChanged_two_files(self):
        file1 = 'test.txt'
        file2 = 'test2.txt'
        self._template('one_task_dependency_4', 'mkfile.py')
        self._import_mkfile()

        self._add_task('/task_21')

        self.touch(file1, None)
        self.touch(file2, None)

        self._pymk_runtask(['task_21'])
        self._pymk_runtask(['task_21'])

        self.touch(file1)
        self._pymk_runtask(['task_21', 'task_21'])

        self.touch(file2)
        self._pymk_runtask(['task_21', 'task_21', 'task_21'])
        self._pymk_runtask(['task_21', 'task_21', 'task_21'])

    def test_graph(self):
        dep = FileChanged('filename')

        self.assertEqual(str, type(dep.extra()))
        self.assertEqual(str, type(dep.get_graph_name()))
        self.assertEqual(str, type(dep.get_graph_details()))
        dep.runned = True

        self.assertEqual(str, type(dep.extra()))
        self.assertEqual(str, type(dep.get_graph_name()))
        self.assertEqual(str, type(dep.get_graph_details()))

        output = StringIO.StringIO()
        dep.write_graph_detailed(output)
        self.assertNotEqual(0, len(output.getvalue().strip()))

    def test_check_force(self):
        dep = FileChanged('filename')
        self.assertTrue(dep.do_test(None, True))

    def test_no_output_file(self):
        class TaskA(Task):
            output_file = 'something'
            dependencys = []

        class TaskB(Task):
            dependencys = []

        dep = FileChanged('filename', TaskB)
        self.assertRaises(Perror.TaskMustHaveOutputFile, dep.do_test, TaskA)

    def test_no_output_file2(self):
        class TaskBa(Task):
            output_file = 'something'
            dependencys = []

        class TaskBb(Task):
            dependencys = []
        dep = FileChanged('filename', TaskBa)
        self.assertRaises(Perror.TaskMustHaveOutputFile, dep.do_test, TaskBb)

    def test_make_dependent_file(self):
        task = MagicMock()
        depedency = FileChanged('name', task)

        self.assertTrue(depedency.make_dependent_file())
        task.run.called_once_with(False)

    def test_CouldNotCreateFile(self):
        depedency = FileChanged('name')
        self.assertRaises(
            Perror.CouldNotCreateFile, depedency.make_dependent_file)


class AlwaysRebuildDependencyTest(PymkTestCase):

    def test_success(self):
        self._template('three_task_dependency_always1', 'mkfile.py')
        self._import_mkfile()
        self._add_task('/task_16a')

        self._pymk_runtask(['task_16c', 'task_16b', 'task_16a'])
        self._pymk_runtask(['task_16c', 'task_16b', 'task_16a', 'task_16a'])
        self._pymk_runtask(
            ['task_16c', 'task_16b', 'task_16a', 'task_16a', 'task_16a'])

    def test_dependency_rebuild(self):
        self._template('three_task_dependency_always1', 'mkfile.py')
        self._import_mkfile()
        self._add_task('/task_16a')

        self._pymk_runtask(['task_16c', 'task_16b', 'task_16a'])
        self.touch('c.out')
        self._pymk_runtask(
            ['task_16c', 'task_16b', 'task_16a', 'task_16b', 'task_16a'])
        self._pymk_runtask(['task_16c', 'task_16b', 'task_16a',
                           'task_16b', 'task_16a', 'task_16a'])

    def test_graph(self):
        dep = AlwaysRebuild()

        self.assertEqual(str, type(dep.extra()))
        self.assertEqual(str, type(dep.get_graph_name()))
        self.assertEqual(str, type(dep.get_graph_details()))

        dep.runned = True

        self.assertEqual(str, type(dep.extra()))
        self.assertEqual(str, type(dep.get_graph_name()))
        self.assertEqual(str, type(dep.get_graph_details()))

        output = StringIO.StringIO()
        dep.write_graph_detailed(output)
        self.assertNotEqual(0, len(output.getvalue().strip()))


class InnerFileExistsTest(PymkTestCase):

    def test_graph(self):
        dep = InnerFileExists(Task)

        self.assertEqual(str, type(dep.extra()))
        self.assertEqual(str, type(dep.get_graph_name()))

        dep.runned = True

        self.assertEqual(str, type(dep.extra()))
        self.assertEqual(str, type(dep.get_graph_name()))


class InnerFileChangedTest(PymkTestCase):

    def test_graph(self):
        dep = InnerFileChanged(Task)

        self.assertEqual(str, type(dep.extra()))
        self.assertEqual(str, type(dep.get_graph_name()))

        dep.runned = True

        self.assertEqual(str, type(dep.extra()))
        self.assertEqual(str, type(dep.get_graph_name()))


class InnerLinkTest(PymkTestCase):

    def setUp(self):
        super(InnerLinkTest, self).setUp()
        self.parent = MagicMock()
        self.dep = InnerLink(self.parent)

    def test_do_test(self):
        self.assertFalse(self.dep.do_test('1'))
        self.parent.run.assert_called_once_with(False, parent='1')

    def test_extra(self):
        self.assertEqual('[label="L"]', self.dep.extra())



import os

from pymk.tests.base import PymkTestCase
from pymk.extra import touch, find_files


class FindFilesTest(PymkTestCase):

    def setUp(self):
        super(FindFilesTest, self).setUp()

        os.mkdir('first')
        os.mkdir('second')
        os.mkdir('third')

        touch('file2.test')
        touch('file1.test')
        touch('file3.notest')

        for filename in ['file3.test', 'file4.test', 'file5.notest']:
            path = os.path.join('first', filename)
            touch(path)

        for filename in ['file6.test', 'file7.test', 'file8.notest']:
            path = os.path.join('second', filename)
            touch(path)

        for filename in ['file9.test', 'file10.test', 'file11.notest']:
            path = os.path.join('third', filename)
            touch(path)

    def test_nothing_found(self):
        self.assertEqual([], list(find_files('.', '*.py')))

    def test_found_all_tests(self):
        should_found = [
            './file1.test',
            './file2.test',
            './third/file10.test',
            './third/file9.test',
            './second/file7.test',
            './second/file6.test',
            './first/file4.test',
            './first/file3.test'
        ]
        self.assertEqual(should_found, list(find_files('.', '*.test')))



import signal

from subprocess import Popen, PIPE

from pymk.error import CommandError, CommandAborted


class SignalHandlingType(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(
                SignalHandlingType, cls).__call__(*args, **kwargs)
        return cls._instances[cls]


class SignalHandling(object):

    __metaclass__ = SignalHandlingType

    def __init__(self):
        self.do_init()

    def do_init(self):
        self.aborted = False
        for _signal in [
            signal.SIGABRT,
            signal.SIGFPE,
            signal.SIGILL,
            signal.SIGINT,
            signal.SIGSEGV,
            signal.SIGTERM
        ]:
            signal.signal(_signal, self.on_signal)

    def send_signal(self, spp, signal):
        try:
            if spp.poll() == None:
                spp.send_signal(signal)
        except OSError:
            pass

    def on_signal(self, signum, frame):
        self.aborted = True
        for spp in Process.all_elements:
            self.send_signal(spp, signum)


class Process(object):
    all_elements = []

    def __init__(self, args, show_output=True, env=None):
        self.prepere_args(args)
        self.show_output = show_output
        self.env = env
        self.run()
        self.append_proccess()
        try:
            self.wait_for_termination()
        finally:
            self.end_proccess()

    def prepere_args(self, args):
        if type(args) in (str,  unicode):
            self.args = [args, ]
        else:
            self.args = args

    def run(self):
        if self.show_output:
            self.spp = Popen(self.args, shell=True, env=self.env)
        else:
            self.spp = Popen(self.args, stdout=PIPE, stderr=PIPE, shell=True, env=self.env)

    def wait_for_termination(self):
        error = self.spp.wait()
        if SignalHandling().aborted:
            raise CommandAborted()
        if error != 0:
            self.raise_error(error)

    def raise_error(self, error):
        if self.show_output:
            raise CommandError(error, '')
        else:
            raise CommandError(error, self.spp.stderr.read())

    def append_proccess(self):
        self.all_elements.append(self.spp)

    def end_proccess(self):
        SignalHandling().send_signal(self.spp, signal.SIGTERM)
        self.all_elements.remove(self.spp)


def run(args, show_output=True, env=None):
    process = Process(args, show_output, env=env)
    return process.spp.stdout, process.spp.stderr



import os
from contextlib import nested
from mock import patch, MagicMock

from pymk.error import RecipeAlreadyExists, WrongPymkVersion
from pymk.recipe import Recipe, RecipeType, DownloadRecipe
from pymk.task import TaskType, Task
from pymk.tests.base import PymkTestCase


class RecipeExample(Recipe):
    singleton = False

    def create_settings(self):
        self.set_path('home', ['/tmp', ])


class TaskExample(Task):
    dependencys = []

    def build(self):
        pass # pragma: no cover


class RecipeTest(PymkTestCase):

    def setUp(self):
        super(RecipeTest, self).setUp()
        self.recipe = RecipeExample()

    def test_set_path(self):
        self.recipe.set_path('path', ['%(home)s', 'elf', 'samantha'])

        self.assertEqual(
            os.path.join('/tmp', 'elf', 'samantha'),
            self.recipe.paths['path']
        )

    def test_set_path_no_list(self):
        self.recipe.set_path('path', '/something')

        self.assertEqual('/something', self.recipe.paths['path'])

    def test_set_setting(self):
        self.recipe.settings['name'] = 'my name'
        self.recipe.set_setting('something', ':%(name)s:')

        self.assertEqual(':my name:', self.recipe.settings['something'])

    def test_getName(self):
        self.assertEqual('pymk.tests.recipe', self.recipe.getName())

    def test_assign_main_path(self):
        self.recipe.assign_main_path(__file__)
        self.assertEqual(
            os.path.dirname(__file__), self.recipe.paths['main'])

    def test_getDefaultTask_no_task(self):
        self.recipe.default_task = None
        task = self.recipe.getDefaultTask()
        self.assertEqual(None, task)

    @patch.object(TaskType, 'tasks', {})
    def test_getDefaultTask(self):
        TaskType.tasks['default'] = MagicMock()
        self.recipe.default_task = 'default'

        task = self.recipe.getDefaultTask()
        self.assertEqual(TaskType.tasks['default'], task)

    @patch('pymk.recipe.compare_version')
    def test_validate_settings_equal(self, compare_version):
        compare_version.return_value = 0
        self.recipe.settings['minimal pymk version'] = MagicMock()
        with patch('pymk.recipe.VERSION') as VERSION:
            self.assertEqual(None, self.recipe.validate_settings())
            compare_version.assert_called_once_with(
                VERSION, self.recipe.settings['minimal pymk version'])

    @patch('pymk.recipe.compare_version')
    def test_validate_settings_greater(self, compare_version):
        compare_version.return_value = 1
        self.recipe.settings['minimal pymk version'] = MagicMock()
        with patch('pymk.recipe.VERSION') as VERSION:
            self.assertEqual(None, self.recipe.validate_settings())
            compare_version.assert_called_once_with(
                VERSION, self.recipe.settings['minimal pymk version'])

    @patch('pymk.recipe.compare_version')
    def test_validate_settings_lower(self, compare_version):
        compare_version.return_value = -1
        self.recipe.settings['minimal pymk version'] = MagicMock()
        with patch('pymk.recipe.VERSION') as VERSION:
            self.assertRaises(WrongPymkVersion, self.recipe.validate_settings)
            compare_version.assert_called_once_with(
                VERSION, self.recipe.settings['minimal pymk version'])

    def test_name(self):
        self.recipe.settings['name'] = 'my name'
        self.assertEqual('my name', self.recipe.name())

    def test_import_wrapper(self):
        with nested(
                patch('__builtin__.globals'),
                patch('__builtin__.locals'),
                patch('__builtin__.__import__'),
        ) as (_globals, _locals, _import):
            self.recipe.import_wrapper('name')
            _import.assert_called_once_with(
                'name', _globals.return_value, _locals.return_value, [''])

    def test_add_recipe(self):
        task = MagicMock()
        with nested(
                patch.dict(RecipeType.recipes, {'pymkmodules.name': task}),
                patch.object(self.recipe, 'import_wrapper'),
        ) as (recipes, import_wrapper):
            self.recipe.add_recipe('name')

            import_wrapper.assert_called_once_with('pymkmodules.name')
            self.assertTrue(task in self.recipe.recipes)
            task.assert_called_once_with()

    def test_add_recipe_no_recipe(self):
        task = MagicMock()
        with nested(
                patch.dict(RecipeType.recipes, {}),
                patch.object(self.recipe, 'import_wrapper'),
        ) as (recipes, import_wrapper):
            self.recipe.add_recipe('name')
            import_wrapper.assert_called_once_with('pymkmodules.name')
            self.assertFalse(task in self.recipe.recipes)
            self.assertEqual(0, task.call_count)


class RecipeTypeTest(PymkTestCase):

    def _init_patchers(self):
        super(RecipeTypeTest, self)._init_patchers()
        self.patchers['recipes'] = patch.object(RecipeType, 'recipes', {})
        self.patchers['args'] = patch.object(RecipeType, 'args', {})
        self.patchers['_instances'] = patch.dict(RecipeType._instances, {})

    def test_init(self):
        with nested(
                patch.object(RecipeType, 'check_if_recipe_exists'),
                patch.object(RecipeType, 'assign_recipe_to_tasks'),
        ) as (check_if_recipe_exists, assign_recipe_to_tasks):
            RecipeType.getName = MagicMock()
            RecipeType('RecipeExample', (), {})

            check_if_recipe_exists.assert_called_once_with('pymk.tests.recipe')
            assign_recipe_to_tasks.assert_called_once()
            self.assertTrue(
                RecipeType.getName.return_value in RecipeType.recipes)

    def test_init_base_class(self):
        with nested(
                patch.object(RecipeType, 'check_if_recipe_exists'),
                patch.object(RecipeType, 'assign_recipe_to_tasks'),
        ) as (check_if_recipe_exists, assign_recipe_to_tasks):
            RecipeType('RecipeExample', (), {'base': True})

            self.assertEqual(0, check_if_recipe_exists.call_count)
            self.assertEqual(0, assign_recipe_to_tasks.call_count)
            self.assertEqual({}, RecipeType.recipes)

    def test_check_if_recipe_exists_true(self):
        RecipeType.recipes['something'] = 1
        self.assertRaises(
            RecipeAlreadyExists, RecipeType.check_if_recipe_exists, 'something')

    def test_check_if_recipe_exists_false(self):
        self.assertEqual(None, RecipeType.check_if_recipe_exists('something'))

    def test_getRecipeForModule(self):
        self.assertEqual(None, RecipeType.getRecipeForModule('something'))

    def test_getRecipeForModule_with_module(self):
        RecipeType.recipes['something'] = 1
        self.assertEqual(1, RecipeType.getRecipeForModule('something'))

    def test_is_not_a_base_class_no_base(self):
        task = object()
        self.assertFalse(RecipeType.is_not_a_base_class(task))

    def test_is_not_a_base_class_base_is_true(self):
        task = MagicMock()
        task.base = True
        self.assertFalse(RecipeType.is_not_a_base_class(task))

    def test_is_not_a_base_class_base_is_false(self):
        task = MagicMock()
        task.base = False
        self.assertTrue(RecipeType.is_not_a_base_class(task))

    def test_assign_recipe_to_task_if_able_do_assign(self):
        task = TaskExample
        recipe = MagicMock()
        with nested(
                patch.object(RecipeType, 'is_not_a_base_class'),
                patch.object(task, 'assign_recipe'),
        ) as (is_not_a_base_class, assign_recipe):
            is_not_a_base_class.return_value = True
            RecipeType.assign_recipe_to_task_if_able(recipe, task)

            is_not_a_base_class.assert_called_once_with(task)
            assign_recipe.assert_called_once_with(recipe)

    def test_assign_recipe_to_task_if_able_base_class(self):
        task = TaskExample
        recipe = MagicMock()
        with nested(
                patch.object(RecipeType, 'is_not_a_base_class'),
                patch.object(task, 'assign_recipe'),
        ) as (is_not_a_base_class, assign_recipe):
            is_not_a_base_class.return_value = False
            RecipeType.assign_recipe_to_task_if_able(recipe, task)

            is_not_a_base_class.assert_called_once_with(task)
            self.assertEqual(0, assign_recipe.call_count)

    def test_assign_recipe_to_task_if_able_wrong_class(self):
        task = TaskExample()
        recipe = MagicMock()
        with nested(
                patch.object(RecipeType, 'is_not_a_base_class'),
                patch.object(task, 'assign_recipe'),
        ) as (is_not_a_base_class, assign_recipe):
            RecipeType.assign_recipe_to_task_if_able(recipe, task)

            self.assertEqual(0, is_not_a_base_class.call_count)
            self.assertEqual(0, assign_recipe.call_count)


class DownloadRecipeTest(PymkTestCase):

    def setUp(self):
        super(DownloadRecipeTest, self).setUp()
        self.downloader = DownloadRecipe('name', 'url')

    @patch('pymk.recipe.DownloadRecipe')
    def test_download_from_recipe(self, DownloadRecipe):
        recipe = RecipeExample()
        recipe.download_recipe('name', 'url')

        DownloadRecipe.assert_called_once_with('name', 'url')
        DownloadRecipe.return_value.run.assert_called_once_with()

    def test_init(self):
        self.assertEqual('name', self.downloader.name)
        self.assertEqual('url', self.downloader.url)

    @patch('pymk.recipe.os.path.exists')
    def test_run_exists(self, existsmock):
        existsmock.return_value = True
        with patch.object(self.downloader, 'create_pymkmodules_path') as create_pymkmodules_path:
            with patch.object(self.downloader, 'download') as download:
                self.downloader.run()

                self.assertEqual(0, download.call_count)
                create_pymkmodules_path.assert_called_once_with()
                existsmock.assert_called_once_with(
                    '%s/%s.egg' % (self.downloader.modules_path, self.downloader.name))

    @patch('pymk.recipe.os.path.exists')
    def test_run_not_exists(self, existsmock):
        existsmock.return_value = False
        with patch.object(self.downloader, 'create_pymkmodules_path') as create_pymkmodules_path:
            with patch.object(self.downloader, 'download') as download:
                self.downloader.run()

                download.assert_called_once_with()
                create_pymkmodules_path.assert_called_once_with()
                existsmock.assert_called_once_with(
                    '%s/%s.egg' % (self.downloader.modules_path, self.downloader.name))

    @patch('pymk.recipe.os.path.exists')
    def test_create_pymkmodules_path_exists(self, existsmock):
        existsmock.return_value = True
        with patch('pymk.recipe.os.mkdir') as mkdirmock:
            self.downloader.create_pymkmodules_path()

            existsmock.assert_called_once_with(self.downloader.modules_path)
            self.assertEqual(0, mkdirmock.call_count)

    @patch('pymk.recipe.os.path.exists')
    def test_create_pymkmodules_path_not_exists(self, existsmock):
        existsmock.return_value = False
        with patch('pymk.recipe.os.mkdir') as mkdirmock:
            self.downloader.create_pymkmodules_path()

            existsmock.assert_called_once_with(self.downloader.modules_path)
            mkdirmock.assert_called_once_with(self.downloader.modules_path)

    @patch('pymk.recipe.log')
    def test_download(self, log):
        with patch('pymk.recipe.download') as download:
            with patch('pymk.recipe.extract_egg') as extract_egg:
                self.downloader.download()

                destination_egg_path = self.downloader.destination_path + \
                    '.zip'

                self.assertEqual(2, log.info.call_count)
                download.assert_called_once_with(
                    self.downloader.url, destination_egg_path)

                extract_egg.assert_called_once_with(
                    destination_egg_path, self.downloader.destination_path)



import os


def touch(path):
    """touch(filename) -> None
    Updates file access and modified times specified by path.
    """
    fhandle = file(path, 'a')
    try:
        os.utime(path, None)
    finally:
        fhandle.close()



from mock import patch, call, MagicMock
from contextlib import nested
from subprocess import PIPE
import signal

import pymk.error as Perror
from pymk import extra
from pymk.tests.base import PymkTestCase
from pymk.error import CommandAborted, CommandError


class RunCmdTest(PymkTestCase):

    def test_success(self):
        ret = extra.run(['ls -al'], False)
        self.assertEqual(file, type(ret[0]))
        self.assertEqual(file, type(ret[1]))

    def test_show_output(self):
        ret = extra.run(['ls', '*.py'], show_output=True)
        self.assertEqual(None, ret[0])
        self.assertEqual(None, ret[1])

    def test_fail(self):
        self.assertRaises(Perror.CommandError, extra.run, ['ls *.py'], False)

    def test_fail_with_show_enabled(self):
        self.assertRaises(
            Perror.CommandError, extra.run, ['ls *.py'], True)

    def test_providing_string(self):
        self.assertRaises(CommandError, extra.run, 'ls *.py', False)

    @patch('pymk.extra.cmd.Process')
    def test_run_function(self, Process):
        ret = extra.run([1, 2, 3], True, 'env')

        Process.assert_called_once_with([1, 2, 3], True, env='env')
        spp = Process.return_value.spp

        self.assertEqual((spp.stdout, spp.stderr), ret)


class SignalHandlingTest(PymkTestCase):

    def setUp(self):
        super(SignalHandlingTest, self).setUp()
        self.handler = extra.cmd.SignalHandling()

    @patch('pymk.extra.cmd.signal')
    def test_init(self, signal):
        self.handler.do_init()

        self.assertEqual(
            call(signal.SIGABRT, self.handler.on_signal), signal.signal.call_args_list[0])
        self.assertEqual(
            call(signal.SIGFPE, self.handler.on_signal), signal.signal.call_args_list[1])
        self.assertEqual(
            call(signal.SIGILL, self.handler.on_signal), signal.signal.call_args_list[2])
        self.assertEqual(
            call(signal.SIGINT, self.handler.on_signal), signal.signal.call_args_list[3])
        self.assertEqual(
            call(signal.SIGSEGV, self.handler.on_signal), signal.signal.call_args_list[4])
        self.assertEqual(
            call(signal.SIGTERM, self.handler.on_signal), signal.signal.call_args_list[5])
        self.assertFalse(self.handler.aborted)

    def test_send_signal(self):
        spp = MagicMock()
        spp.poll.return_value = None
        mysignal = MagicMock()
        self.handler.send_signal(spp, mysignal)

        spp.send_signal.assert_called_once(mysignal)

    def test_send_signal_with_oserror(self):
        spp = MagicMock()
        spp.poll.return_value = None
        spp.send_signal.side_effect = OSError()
        mysignal = MagicMock()
        self.handler.send_signal(spp, mysignal)

        spp.send_signal.assert_called_once(mysignal)

    @patch('pymk.extra.cmd.Process')
    def test_on_signal(self, Process):
        Process.all_elements = [MagicMock()]
        signum = MagicMock()
        frame = MagicMock()

        self.handler.aborted = False
        with patch.object(self.handler, 'send_signal') as send_signal:
            self.handler.on_signal(signum, frame)

            send_signal.assert_called_once_with(
                Process.all_elements[0], signum)
            self.assertTrue(self.handler.aborted)

        self.handler.aborted = False


class ProccessTest(PymkTestCase):

    def setUp(self):
        super(ProccessTest, self).setUp()
        extra.cmd.Process.all_elements = []
        with patch.object(extra.cmd.Process, '__init__', return_value=None):
            self.proccess = extra.cmd.Process()
            self.proccess.spp = MagicMock()

    def tearDown(self):
        super(ProccessTest, self).tearDown()
        extra.cmd.Process.all_elements = []

    def test_init(self):
        with nested(
                patch.object(extra.cmd.Process, 'run'),
                patch.object(extra.cmd.Process, 'wait_for_termination'),
                patch.object(extra.cmd.Process, 'append_proccess'),
                patch.object(extra.cmd.Process, 'end_proccess'),
        ) as (run, wait_for_termination, append_proccess, end_proccess):
            process = extra.cmd.Process('something')
            self.assertEqual(['something', ], process.args)
            self.assertTrue(process.show_output)
            run.assert_called_once_with()
            append_proccess.assert_called_once_with()
            end_proccess.assert_called_once_with()

    def test_init_with_exception(self):
        with nested(
                patch.object(extra.cmd.Process, 'run'),
                patch.object(extra.cmd.Process, 'wait_for_termination'),
                patch.object(extra.cmd.Process, 'append_proccess'),
                patch.object(extra.cmd.Process, 'end_proccess'),
        ) as (run, wait_for_termination, append_proccess, end_proccess):
            wait_for_termination.side_effect = Exception('Boom!')

            self.assertRaises(Exception, extra.cmd.Process, 'something')
            run.assert_called_once_with()
            append_proccess.assert_called_once_with()
            end_proccess.assert_called_once_with()

    def test_prepere_args(self):
        self.proccess.prepere_args(['args', 'args2'])
        self.assertEqual(['args', 'args2'], self.proccess.args)

    def test_prepere_args_no_list(self):
        self.proccess.prepere_args('args')
        self.assertEqual(['args'], self.proccess.args)

    @patch('pymk.extra.cmd.Popen')
    def test_run(self, Popen):
        self.proccess.args = 'args'
        self.proccess.show_output = False
        self.proccess.env = None

        self.proccess.run()

        Popen.assert_called_once_with(
            'args', stdout=PIPE, stderr=PIPE, shell=True, env=None)

    @patch('pymk.extra.cmd.Popen')
    def test_run_with_output(self, Popen):
        self.proccess.args = 'args'
        self.proccess.show_output = True
        self.proccess.env = 'env'

        self.proccess.run()

        Popen.assert_called_once_with('args', shell=True, env='env')

    @patch('pymk.extra.cmd.SignalHandling')
    def test_wait_for_termination_no_error(self, SignalHandling):
        SignalHandling.return_value.aborted = False
        self.proccess.spp.wait.return_value = 0
        with patch.object(self.proccess, 'raise_error') as raise_error:
            self.proccess.wait_for_termination()
            self.assertEqual(0, raise_error.call_count)

    @patch('pymk.extra.cmd.SignalHandling')
    def test_wait_for_termination_with_error(self, SignalHandling):
        SignalHandling.return_value.aborted = False
        self.proccess.spp.wait.return_value = 5
        with patch.object(self.proccess, 'raise_error') as raise_error:
            self.proccess.wait_for_termination()
            raise_error.assert_called_once_with(5)

    @patch('pymk.extra.cmd.SignalHandling')
    def test_wait_for_termination_with_abort(self, SignalHandling):
        SignalHandling.return_value.aborted = True
        self.proccess.spp.wait.return_value = 5
        with patch.object(self.proccess, 'raise_error') as raise_error:
            self.assertRaises(
                CommandAborted, self.proccess.wait_for_termination)
            self.assertEqual(0, raise_error.call_count)

    @patch('pymk.extra.cmd.CommandError')
    def test_raise_error_with_output(self, CommandError):
        self.proccess.show_output = True
        CommandError.return_value = Exception()

        self.assertRaises(Exception, self.proccess.raise_error, 5)
        CommandError.assert_called_once_with(5, '')

    @patch('pymk.extra.cmd.CommandError')
    def test_raise_error_without_output(self, CommandError):
        self.proccess.show_output = False
        CommandError.return_value = Exception()

        self.assertRaises(Exception, self.proccess.raise_error, 5)
        CommandError.assert_called_once_with(
            5, self.proccess.spp.stderr.read.return_value)

    def test_append_proccess(self):
        self.proccess.append_proccess()

        self.assertEqual([self.proccess.spp, ], self.proccess.all_elements)

    @patch('pymk.extra.cmd.SignalHandling')
    def test_end_proccess(self, SignalHandling):
        self.proccess.all_elements.append(self.proccess.spp)

        self.proccess.end_proccess()

        SignalHandling.return_value.send_signal.assert_called_once_with(
            self.proccess.spp, signal.SIGTERM)
        self.assertEqual([], self.proccess.all_elements)



import sys

import logging
import os
from glob import glob
from smallsettings import Settings, Paths

from pymk import VERSION, compare_version
from pymk.dependency import Dependency
from pymk.download import download, extract_egg
from pymk.error import WrongPymkVersion, BadTaskPath, NotADependencyError, RecipeAlreadyExists
from pymk.task import Task


log = logging.getLogger('pymk')


class RecipeType(type):
    recipes = {}
    args = {}
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls.singleton:
            if cls not in cls._instances:
                cls._instances[cls] = super(
                    RecipeType, cls).__call__(*args, **kwargs)
            return cls._instances[cls]
        else:
            return super(RecipeType, cls).__call__(*args, **kwargs)

    def __init__(cls, name, bases, dct):
        if not 'base' in dct or not dct['base']:
            RecipeType.check_if_recipe_exists(cls.__module__)
            RecipeType.recipes[cls.getName()] = cls
            RecipeType.assign_recipe_to_tasks(cls)

    @classmethod
    def check_if_recipe_exists(cls, name):
        if name in list(cls.recipes):
            raise RecipeAlreadyExists(name)

    @classmethod
    def is_not_a_base_class(cls, task):
        return hasattr(task, 'base') and not task.base

    @classmethod
    def assign_recipe_to_task_if_able(cls, recipe, task):
        try:
            if issubclass(task, Task) and cls.is_not_a_base_class(task):
                task.assign_recipe(recipe)
        except TypeError:
            # class do not have a base value
            pass

    @classmethod
    def assign_recipe_to_tasks(cls, recipe):
        #----------------------------------------------------------------------
        module = sys.modules[recipe.__module__]
        for name in dir(module):
            task = getattr(module, name)
            cls.assign_recipe_to_task_if_able(recipe, task)

    @classmethod
    def getRecipeForModule(cls, name):
        return RecipeType.recipes.get(name)


class Recipe(object):

    default_task = None
    base = True
    __metaclass__ = RecipeType
    singleton = True

    def __init__(self):
        self.settings = Settings({
            'minimal pymk version': VERSION,
            'version': '4.0.0',
        })
        self.paths = Paths()
        self.recipes = []

        self.create_settings()
        self.validate_settings()
        self.gather_recipes()
        self.refresh_modules()
        self.validate_pymkmodules()
        self.validate_tasks()

    def validate_pymkmodules(self):
        if os.path.exists('pymkmodules') and not os.path.exists('pymkmodules/__init__.py'):
            open('pymkmodules/__init__.py', 'a').close()

    def assign_main_path(self, path):
        self.paths['main'] = os.path.dirname(path)

    @classmethod
    def getName(cls):
        return cls.__module__

    def getTask(self, path):
        self._getTaskType().get_task(path)

    def _getTaskType(self):
        from pymk.task import TaskType
        return TaskType

    def getDefaultTask(self):
        if self.default_task is None:
            return None
        else:
            try:
                return self._getTaskType().get_task(self.default_task)
            except KeyError:
                raise BadTaskPath(self.default_task)

    def validate_settings(self):
        min_pymk_version = self.settings['minimal pymk version']
        if compare_version(VERSION, min_pymk_version) == -1:
            raise WrongPymkVersion(VERSION, min_pymk_version)

    def create_settings(self):
        pass

    def set_setting(self, name, value):
        parsed_value = value % self.settings
        self.settings[name] = parsed_value

    def set_path(self, name, values):
        if type(values) not in (list, tuple):
            values = [values, ]
        parsed_values = []
        for value in values:
            parsed_values.append(
                value % self.paths
            )
        self.paths[name] = os.path.join(*parsed_values)

    def gather_recipes(self):
        pass

    def refresh_modules(self):
        sys.path[0:0] = ['pymkmodules'] + glob('pymkmodules/*.egg')

    def download_recipe(self, name, url):
        download_recipe = DownloadRecipe(name, url)
        return download_recipe.run()

    def import_wrapper(self, name):
        return __import__(name, globals(), locals(), [''])

    def add_recipe(self, name):
        name = '.'.join(['pymkmodules', name])
        self.import_wrapper(name)
        if name in RecipeType.recipes:
            recipe = RecipeType.recipes[name]
            recipe()
            self.recipes.append(recipe)

    def name(self):
        return self.settings['name']

    def validate_tasks(self):
        for path, task in self._getTaskType().tasks.items():
            for dependency in task().dependencys:
                if not issubclass(type(dependency), Dependency):
                    raise NotADependencyError(dependency, task)


class DownloadRecipe(object):
    modules_path = 'pymkmodules'

    def __init__(self, name, url):
        self.name = name
        self.url = url
        self.destination_path = os.path.join(
            self.modules_path, self.name + '.egg')

    def run(self):
        self.create_pymkmodules_path()
        if not os.path.exists(self.destination_path):
            self.download()

    def create_pymkmodules_path(self):
        if not os.path.exists(self.modules_path):
            os.mkdir(self.modules_path)

    def download(self):
        destination_egg_path = self.destination_path + '.zip'
        log.info("Downloading %s..." % (self.name,))
        download(self.url, destination_egg_path)
        log.info("Extracting %s..." % (self.name,))
        extract_egg(destination_egg_path, self.destination_path)



from jinja2 import Environment, PackageLoader


class Template(object):
    cache = {}  # yes, this should be "static"

    def get_env(self):
        if not 'env' in self.cache:
            loader = PackageLoader('pymktemplates', '.')
            self.cache['env'] = Environment(loader=loader)
        return self.cache['env']

    def make(self, template_path, output_file, data={}):
        env = self.get_env()
        template = env.get_template(template_path)
        stream = template.stream(**data)
        stream.dump(output_file)


def mktemplate(template_path, output_file, data={}):
    return Template().make(template_path, output_file, data)



import sys
import argparse
import logging
import os

from pymk import VERSION
from pymk import error
from pymk.extra.cmd import SignalHandling
from pymk.graph import draw_graph
from pymk.recipe import RecipeType
from pymk.task import TaskType


log = logging.getLogger('pymk')


def make_graph(args, is_graphviz):
    from pymk.graph import draw_done_task_graph

    def make_task_list():
        if len(args.task) == 0:
            return False
        else:
            try:
                return [TaskType.tasks[task] for task in args.task]
            except KeyError:
                return False
        return tasks
    #-------------------------------------------------------------------------
    if is_graphviz:
        tasks = make_task_list()

        if tasks:
            draw_done_task_graph(args.graph, tasks)
        else:
            draw_graph(args.graph)
    else:
        log.error('ERROR: No dot executable found!')


def check_for_graphviz(args):
    from pymk.extra import run
    if args.graph:
        try:
            run('which dot')
            return True
        except error.CommandError:
            pass
    return False


def append_python_path(cwd=None):
    if cwd:
        sys.path.append(cwd)
    else:
        sys.path.append(os.getcwd())


def import_mkfile(name='mkfile'):
    if not os.path.exists('%s.py' % (name,)):
        raise error.NoMkfileFound()
    if 'mkfile' in sys.modules:
        module = __import__(name, globals(), locals())
        reload(module)
    else:
        module = __import__(name, globals(), locals())
    return module


def init_recipe(name):
    recipe = RecipeType.getRecipeForModule(name)
    if recipe:
        recipe()
        return True
    return False


def run_all_inputet_tasks(urls, force, dependency_force):
    def gather_tasks(urls):
        tasks = []

        for url in urls:
            tasks.append(TaskType.get_task(url))
        return tasks

    tasks = gather_tasks(urls) #this needs to be 2 phased
    for task in tasks:
        if not task._get_runned():
            task.run(
                force=force,
                dependency_force=dependency_force,
            )
    return 'run tasks'


def run_tasks(mkfile, args):
    def list_all_tasks():
        text = 'Avalible tasks:\n'
        task_names_size = 4
        task_path_size = 4
        for name, task in TaskType.tasks.items():
            if not task.hide:
                if len(task.getName()) > task_names_size:
                    task_names_size = len(task.getName())
                if len(task.getPath()) > task_path_size:
                    task_path_size = len(task.getPath())
        task_names_size += 2
        task_path_size += 2
        template = '  %-' + \
            str(task_names_size) + 's %-' + str(task_path_size) + 's %s\n'
        text += template % ('Name', 'Path', 'Help')
        text += template % ('----', '----', '----')
        for name, task in TaskType.tasks.items():
            if not task.hide:
                text += template % (
                    task.getName(),
                    task.getPath(),
                    task.help,
                )

        log.info(text)
        return 'list all'

    def run_default_task_or_list_all_tasks():
        try:
            if args.graph:
                return 'do graph of all'
            else:
                recipe = RecipeType.getRecipeForModule('mkfile')
                task = recipe().getDefaultTask()
                task.run(
                    force=args.force, dependency_force=args.dependency_force)
                return 'run default'
        except (AttributeError, TypeError):
            return list_all_tasks()
    #-----------------------------------------------------------------------
    if args.all:
        return list_all_tasks()
    elif len(args.task) == 0:
        return run_default_task_or_list_all_tasks()
    else:
        return run_all_inputet_tasks(
            args.task,
            args.force,
            args.dependency_force
        )


def show_tasks_paths(arg_path):
    for name, task in TaskType.tasks.items():
        if task.getPath().startswith(arg_path):
            print task.getPath()


def run():
    """run() -> error code
    Parse command line args and do the task provided. More info with inputing "pymk -h"

    @return:
     1. no mkfile.py found or it is corrupted
     2. error in external program
     3. wrong task name
     4. provided task has no output_file, which is needed becouse of dependencys
     5. could not create a file that is in depedency
     6. command aborted (by keyboard)
    """
    def parse_command():
        parser = argparse.ArgumentParser()
        parser.add_argument('task', nargs='*',
                            help='List of task to do.')
        parser.add_argument('-l', '--log', dest='log', default='info',
                            help='Set log level from "debug" or "info".')
        parser.add_argument('-a', '--all', dest='all', action='store_true',
                            help='Show all tasks avalible.')
        parser.add_argument('-f', '--force', dest='force', action='store_true',
                            help='Force task to rebuild.')
        parser.add_argument('-g', '--graph', dest='graph',
                            help='Draw a graph of tasks to a file.')
        parser.add_argument('-p', '--paths', dest='paths', const='', nargs='?',
                            help='Show tasks paths')
        parser.add_argument(
            '-v', '--version', dest='version', action='store_true',
            help='Show version of pymk.')
        parser.add_argument(
            '-d', '--dependency-force', dest='dependency_force', action='store_true',
            help='Force depedency to rebuild (use only with --force).')
        return parser.parse_args()

    def start_loggin(args):
        FORMAT = '%(message)s'
        if args.log == 'debug':
            logging.basicConfig(level=logging.DEBUG, format=FORMAT)
        elif args.log == 'info':
            logging.basicConfig(level=logging.INFO, format=FORMAT)
        else:
            raise error.WrongArgumentValue(
                'Wrong argument for log! Avalible are: "debug" and "info".')
    #-------------------------------------------------------------------------

    try:
        SignalHandling()
        args = parse_command()
        if args.version:
            args.log = 'info'
            start_loggin(args)
            log.info('Pymk version %s' % (VERSION,))
            return 0
        else:
            start_loggin(args)
            append_python_path()
            module = import_mkfile('mkfile')
            init_recipe('mkfile')
            is_graphviz = check_for_graphviz(args)
    except error.NoMkfileFound as er:
        log.error("No mkfile.py file found!")
        return 1
    except error.NotADependencyError as er:
        log.error(er)
        return 7

    if args.paths is not None:
        start_loggin(args)
        show_tasks_paths(args.paths)
        return 0
    else:
        try:
            run_tasks(module, args)
        except error.CommandError as er:
            log.error(er)
            return 2
        except error.BadTaskPath as er:
            log.error(er)
            return 3
        except error.TaskMustHaveOutputFile as er:
            log.error(er)
            return 4
        except error.CouldNotCreateFile as er:
            log.error(er)
            return 5
        except (KeyboardInterrupt, error.CommandAborted) as er:
            log.error('\rCommand aborted!')
            return 6
        finally:
            if args.graph:
                make_graph(args, is_graphviz)

if __name__ == '__main__':
    run()



from pymk.task import TaskType
from pymk.dependency import AlwaysRebuild
from tempfile import TemporaryFile, NamedTemporaryFile
from subprocess import Popen

datalog = TemporaryFile('wr')
running_list = []


def run_dot(pipe, filename):
    filepipe = open(filename, 'w')
    spp = Popen(['dot', '-x', '-Tpng'], stdin=pipe, stdout=filepipe)
    spp.wait()


def draw_graph(filename):
    datalog = NamedTemporaryFile('wr', delete=False)
    datalog.write('digraph {\n')
    for task in TaskType.tasks.values():
        for dep in task.dependencys:
            if type(dep) == AlwaysRebuild:
                continue
            dep.write_graph_detailed(datalog)
            datalog.write('"%s" -> "%s" %s;\n' % (dep.name, task.getName(), dep.extra()))
        task.write_graph_detailed(datalog)

    datalog.write('}\n')
    datalog.seek(0)
    run_dot(datalog, filename)


def draw_done_task_graph(filename, tasks):
    def write_task_to_datalog(datalog, task, parent=None):
        if hasattr(task, 'dependencys'):
            for dep in task().dependencys:
                if type(dep) == AlwaysRebuild:
                    continue
                if hasattr(dep, 'parent'):
                    write_task_to_datalog(datalog, dep.parent)
                else:
                    dep.write_graph_detailed(datalog)
                datalog.write('"%s" -> %s %s;\n' % (dep.name, task.getName(), dep.extra()))
        task.write_graph_detailed(datalog)
    #---------------------------------------------------------------------------
    datalog = NamedTemporaryFile('wr', delete=False)
    datalog.write('digraph {\n')
    for task in tasks:
        write_task_to_datalog(datalog, task)

    datalog.write('}\n')
    datalog.seek(0)
    run_dot(datalog, filename)



import logging
import os
from urlparse import urlparse, parse_qs

from pymk.dependency import InnerFileExists, InnerFileChanged, AlwaysRebuild
from pymk.dependency import Dependency, InnerLink
from pymk.error import TaskAlreadyExists, NoDependencysInAClass
from pymk.error import NotADependencyError, BadTaskPath


logger = logging.getLogger('pymk')


class _GetTask(object):

    def __init__(self, url):
        self.url = url

    def parse_url(self):
        url = urlparse(self.url)
        path = url.path
        args = parse_qs(url.query)
        return path, args

    def check_if_task_name_exists(self, path):
        if not path in TaskType.tasks:
            raise BadTaskPath(path)

    def get_task(self, path):
        return TaskType.tasks[path]

    def __call__(self):
        path, args = self.parse_url()
        self.check_if_task_name_exists(path)
        task = self.get_task(path)
        task._args = args
        return task


class TaskType(type):
    tasks = {}
    args = {}
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if getattr(cls, '_no_singleton', False):
            return super(TaskType, cls).__call__(*args, **kwargs)
        if cls not in cls._instances:
            cls._instances[cls] = super(
                TaskType, cls).__call__(*args, **kwargs)
        return cls._instances[cls]

    @classmethod
    def check_if_task_exists(cls, name):
        if name in list(cls.tasks):
            raise TaskAlreadyExists(name)

    @classmethod
    def assign_recipe_if_able(basecls, cls):
        from pymk.recipe import RecipeType
        recipe = RecipeType.getRecipeForModule(cls.__module__)
        if recipe:
            cls.assign_recipe(recipe)

    @classmethod
    def get_task(self, url):
        return _GetTask(url)()

    def __init__(cls, name, bases, dct):
        def validate_dependency(cls):
            if cls.dependencys is None:
                raise NoDependencysInAClass(cls)
            if cls.recipe is None:
                for dependency in cls().dependencys:
                    if not issubclass(type(dependency), Dependency):
                        raise NotADependencyError(dependency, cls)
        #----------------------------------------------------------------------
        if not 'base' in dct or not dct['base']:
            TaskType.assign_recipe_if_able(cls)
            TaskType.check_if_task_exists(cls().getPath())
            TaskType.tasks[cls().getPath()] = cls
            cls.base = False
            validate_dependency(cls)

    @classmethod
    def init(cls):
        """Init new task collection."""
        cls.tasks = {}
        cls.args = {}


class Task(object):

    """Base of all taks."""

    base = True

    __metaclass__ = TaskType

    dependencys = None
    output_file = None
    help = ''
    hide = False
    hide_graph = False

    name = None
    path = None
    detailed = []
    _runned = False
    _args = {}
    _error = False
    recipe = None

    @classmethod
    def getName(cls):
        """getName() -> str
        Returns name of the tasks provided by class value name, or just
        classname if name == None.
        """
        if cls.name:
            return cls.name
        else:
            return cls.__name__

    @classmethod
    def getPath(cls):
        """getPath(cls) -> str
        Returns path (for using in command line) of the tasks provided by class
        value path, or just classname if path == None.
        """
        if cls.path:
            return cls.path
        else:
            return '/' + cls.__name__.lower()

    @classmethod
    def _set_runned(cls, value):
        cls._runned = value
        return cls._runned

    @classmethod
    def _get_runned(cls):
        try:
            return cls._runned
        except KeyError:
            return False

    @classmethod
    def test_dependencys(cls, dependency_force=False):
        """test_dependencys() -> bool

        """
        make_rebuild = False
        for dependency in cls().dependencys:
            cond = dependency(cls, dependency_force=dependency_force)
            make_rebuild = make_rebuild or cond
            cls.running_list_element['childs'].append({
                'type': 'dependency',
                'data': dependency,
                'runned': cond,
                'childs': [],
            })
        if make_rebuild:
            return make_rebuild
        else:
            if cls().output_file:
                if not os.path.exists(cls().output_file):
                    return True
                else:
                    return False
            if len(cls().dependencys) == 0:
                return True
            return False

    @classmethod
    def run(cls,
            log_uptodate=True,
            force=False,
            dependency_force=False,
            parent=None):
        """run(log_uptodate = True): -> bool
        Test dependency of this task, and rebuild it if nessesery.
        """
        def build_with_args_or_not(cls):
            try:
                cls().build(cls._args)
            except TypeError:
                cls().build()
        #----------------------------------------------------------------------
        cls.running_list_element = {
            'type': 'task',
            'data': cls,
            'runned': None,
            'childs': [],
        }
        if parent:
            parent.running_list_element[
                'childs'].append(cls.running_list_element)
        else:
            from pymk.graph import running_list
            running_list.append(cls.running_list_element)
        if cls.test_dependencys(force and dependency_force) or force:
            logger.info(" * Building '%s'" % (cls.getName()))
            try:
                build_with_args_or_not(cls)
            except Exception:
                cls._error = True
                raise
            finally:
                runned = cls._set_runned(True)
            return runned
        else:
            if log_uptodate:
                logger.info(" * '%s' is up to date" % (cls.getName()))
            return cls._set_runned(False)

    @classmethod
    def assign_recipe(cls, recipe):
        cls.recipe = recipe

    @property
    def settings(self):
        return self.recipe().settings

    @property
    def paths(self):
        return self.recipe().paths

    @classmethod
    def dependency_FileExists(cls):
        return InnerFileExists(cls)

    @classmethod
    def dependency_FileChanged(cls):
        return InnerFileChanged(cls)

    @classmethod
    def dependency_Link(cls):
        return InnerLink(cls)

    # -- graph specyfic --
    @classmethod
    def write_graph_detailed(cls, datalog):
        if not cls.hide_graph and not cls.name in cls.detailed:
            cls.detailed.append(cls.getName())
            datalog.write('"%s" [%s];\n' %
                          (cls.getName(), cls.get_graph_details()))

    @classmethod
    def get_graph_details(cls):
        shape = 'box'
        color = 'white'
        types = [type(dependency) for dependency in cls().dependencys]
        if AlwaysRebuild in types:
            shape = 'circle'
            color = 'grey'
        if cls._get_runned():
            color = 'green'
        if cls._error:
            color = 'red'

        return 'shape=%s, regular=1,style=filled,fillcolor=%s' % (shape, color)

    def get_task(self, url):
        return TaskType.get_task(url)



import unittest
import os
import sys
import shutil
import tempfile
import logging
from time import sleep

from pymk.script import import_mkfile, run_tasks, TaskType, init_recipe
from pymk.extra import touch
from pymk.task import Task
from pymk.recipe import RecipeType


class ArgsMockup(object):

    def __init__(self):
        self.all = False
        self.task = []
        self.force = False
        self.dependency_force = False
        self.graph = False


class PymkTestCase(unittest.TestCase):

    def setUp(self):
        self._normal_path = os.getcwd()
        self._actual_path = tempfile.mkdtemp()
        sys.path.append(self._actual_path)
        os.chdir(self._actual_path)
        self._args = ArgsMockup()
        logging.getLogger('pymk').info('=== (%s) %s ===' % (
            self.__class__.__name__, self._testMethodName))
        self.patchers = {}
        self.mocks = {}

        self._init_patchers()
        self._start_patchers()

    def _init_patchers(self):
        pass

    def _start_patchers(self):
        for name, patcher in self.patchers.items():
            self.mocks[name] = patcher.start()

    def tearDown(self):
        sys.path.remove(self._actual_path)
        os.chdir(self._normal_path)
        shutil.rmtree(self._actual_path)
        self._actual_path = None
        if 'mkfile' in sys.modules:
            del sys.modules['mkfile']
            del self._mkfile
        self._mkfile = None
        for name, patcher in self.patchers.items():
            patcher.stop()

    def _template(self, name, out_path=None, vars={}):
        template_path = os.path.join(
            self._normal_path, 'pymk', 'tests', 'tmpl', name + '.tpl')
        template = open(template_path).read()
        out_data = template % vars
        out_file = open(out_path, 'w')
        out_file.write(out_data)
        out_file.close()

    def _import_mkfile(self):
        TaskType.init()
        RecipeType.recipes = {}
        self._mkfile = import_mkfile()
        init_recipe('mkfile')

    def _pymk(self):
        def reset_all_task_runned_flags():
            for name, task in TaskType.tasks.items():
                task._set_runned(False)
        reset_all_task_runned_flags()
        return run_tasks(self._mkfile, self._args)

    def _pymk_runtask(self, output_file):
        self.assertEqual(self._pymk(), 'run tasks')
        self._check_output_file(output_file)

    def _check_output_file(self, test_data):
        try:
            data = open('a.out').read().strip().split('\n')
        except IOError:
            data = []
        self.assertEqual(data, test_data)

    def _add_task(self, task):
        self._args.task.append(task)

    def _remove_task(self, task):
        self._args.task.remove(task)

    def _set_task(self, task, expected_tasks):
        self._args.task = [task]
        self._pymk_runtask(expected_tasks)

    def touch(self, path, wait=0.01):
        if wait:
            sleep(wait)
        touch(path)

class BaseTestTask(Task):
    log_file = 'a.out'

    dependencys = []

    def build(self, args=[]):
        fp = open(self.log_file, 'a')
        fp.write(self.getName())
        fp.write('\n')
        fp.close()



import os
import fnmatch


def find_files(directory, pattern):
    for root, dirs, files in os.walk(directory):
        for basename in files:
            if fnmatch.fnmatch(basename, pattern):
                filename = os.path.join(root, basename)
                yield filename



from mock import patch, MagicMock
import os

from pymk.tests.base import PymkTestCase
import pymk.error as Perror
from pymk.script import import_mkfile
from pymk.task import Task, TaskType
from pymk.error import TaskAlreadyExists, CouldNotCreateFile
from pymk.recipe import RecipeType


class TaskTestExample(Task):
    _no_singleton = True
    dependencys = []


class TaskTestExample2(Task):
    _no_singleton = True
    dependencys = []

    path = '/something'


class TaskTypeTest(PymkTestCase):

    @patch.dict(TaskType.tasks, {'name1': 1})
    def test_check_if_task_exists_fail(self):
        self.assertRaises(
            TaskAlreadyExists, TaskType.check_if_task_exists, 'name1')

    @patch.object(TaskType, 'tasks', {'name1': 1})
    def test_check_if_task_exists(self):
        self.assertEqual(None, TaskType.check_if_task_exists('name2'))

    @patch.object(RecipeType, 'recipes', {})
    def test_assign_recipe_if_able_fail(self):
        cls = MagicMock()
        cls.__module__ = 'name1'

        TaskType.assign_recipe_if_able(cls)
        self.assertEqual(0, cls.assign_recipe.call_count)

    @patch.object(RecipeType, 'recipes', {'name1': 1})
    def test_assign_recipe_if_able(self):
        cls = MagicMock()
        cls.__module__ = 'name1'

        TaskType.assign_recipe_if_able(cls)
        cls.assign_recipe.called_once_with(1)


class TaskTest(PymkTestCase):

    def test_no_mkfile(self):
        self.assertRaises(Perror.NoMkfileFound, import_mkfile)

    def test_empty_mkfile(self):
        self._template('empty', 'mkfile.py')
        self._import_mkfile()

        self.assertEqual(self._pymk(), 'list all')

    def test_bad_task_name(self):
        self._template('empty', 'mkfile.py')
        self._import_mkfile()
        self._add_task('/bad_task_name')

        self.assertRaises(Perror.BadTaskPath, self._pymk)

    def test_default_task(self):
        self._template('one_task_with_default', 'mkfile.py')
        self._import_mkfile()

        self.assertEqual(self._pymk(), 'run default')
        self._check_output_file(['task_0'])

    def test_no_dependency(self):
        self._template('one_task', 'mkfile.py')
        self._import_mkfile()
        self._add_task('/task_1')

        self._pymk_runtask(['task_1'])
        self._pymk_runtask(['task_1', 'task_1'])
        self._pymk_runtask(['task_1', 'task_1', 'task_1'])

    def test_no_dependency_with_outputfile(self):
        self._template('one_task_with_outputfile', 'mkfile.py')
        self._import_mkfile()
        self._add_task('/task_5')

        self._pymk_runtask(['task_5'])
        self._pymk_runtask(['task_5'])
        self.touch('a.out')
        self._pymk_runtask(['task_5'])

    def test_bad_task(self):
        self._template('one_task_bad', 'mkfile.py')
        self.assertRaises(Perror.NoDependencysInAClass, self._import_mkfile)

    def test_NotADependencyError(self):
        self._template('one_task_bad_depedency', 'mkfile.py')
        self.assertRaises(Perror.NotADependencyError, self._import_mkfile)

    def test_NotADependencyError_with_recipe(self):
        self._template('one_task_bad_depedency_with_recipe', 'mkfile.py')
        self.assertRaises(Perror.NotADependencyError, self._import_mkfile)


class TaskForcing(PymkTestCase):

    def test_forcing_file_changed(self):
        self._template('three_task_with_force', 'mkfile.py')
        self._import_mkfile()
        self._add_task('/task_17a')

        self._pymk_runtask(['task_17c', 'task_17b', 'task_17a'])
        self._pymk_runtask(['task_17c', 'task_17b', 'task_17a'])

        self._args.force = True
        self._pymk_runtask(['task_17c', 'task_17b', 'task_17a', 'task_17a'])
        self._pymk_runtask(
            ['task_17c', 'task_17b', 'task_17a', 'task_17a', 'task_17a'])

    def test_dependency_forcing_file_changed(self):
        self._template('three_task_with_force', 'mkfile.py')
        self._import_mkfile()
        self._add_task('/task_17a')

        self._args.dependency_force = True
        self._pymk_runtask(['task_17c', 'task_17b', 'task_17a'])
        self._pymk_runtask(['task_17c', 'task_17b', 'task_17a'])

        self._args.force = True
        self._pymk_runtask(['task_17c', 'task_17b', 'task_17a',
                           'task_17c', 'task_17b', 'task_17a'])
        self._pymk_runtask(['task_17c', 'task_17b', 'task_17a', 'task_17c',
                           'task_17b', 'task_17a', 'task_17c', 'task_17b', 'task_17a'])

    def test_forcing_file_exist(self):
        self._template('three_task_with_force_2', 'mkfile.py')
        self._import_mkfile()
        self._add_task('/task_18a')

        self._pymk_runtask(['task_18c', 'task_18b', 'task_18a'])
        self._pymk_runtask(['task_18c', 'task_18b', 'task_18a'])

        self._args.force = True
        self._pymk_runtask(['task_18c', 'task_18b', 'task_18a', 'task_18a'])
        self._pymk_runtask(
            ['task_18c', 'task_18b', 'task_18a', 'task_18a', 'task_18a'])

    def test_dependency_forcing_file_exist(self):
        self._template('three_task_with_force_2', 'mkfile.py')
        self._import_mkfile()
        self._add_task('/task_18a')

        self._args.dependency_force = True
        self._pymk_runtask(['task_18c', 'task_18b', 'task_18a'])
        self._pymk_runtask(['task_18c', 'task_18b', 'task_18a'])

        self._args.force = True
        self._pymk_runtask(['task_18c', 'task_18b', 'task_18a',
                           'task_18c', 'task_18b', 'task_18a'])
        self._pymk_runtask(['task_18c', 'task_18b', 'task_18a', 'task_18c',
                           'task_18b', 'task_18a', 'task_18c', 'task_18b', 'task_18a'])


class TaskDependencyFileExistsTest(PymkTestCase):

    def test_make_no_outputfile(self):
        self._template('three_task_dependency_exists2', 'mkfile.py')
        self._import_mkfile()
        self._add_task('/task_9a')

        self._pymk_runtask(['task_9c', 'task_9b', 'task_9a'])
        self._pymk_runtask(['task_9c', 'task_9b', 'task_9a'])
        self.touch('b.out')
        self._pymk_runtask(['task_9c', 'task_9b', 'task_9a'])

    def test_make_no_outputfile_fail1(self):
        self._template('three_task_dependency_exists3', 'mkfile.py')
        self._import_mkfile()
        self._add_task('/task_10a')

        self.assertRaises(Perror.TaskMustHaveOutputFile, self._pymk)
        self._check_output_file([])

        self.assertRaises(Perror.TaskMustHaveOutputFile, self._pymk)
        self._check_output_file([])

        self.touch('b.out')
        self.assertRaises(Perror.TaskMustHaveOutputFile, self._pymk)
        self._check_output_file([])

    def test_make_no_outputfile_fail2(self):
        self._template('three_task_dependency_exists4', 'mkfile.py')
        self._import_mkfile()
        self._add_task('/task_11a')

        self.assertRaises(Perror.TaskMustHaveOutputFile, self._pymk)
        self._check_output_file([])

        self.assertRaises(Perror.TaskMustHaveOutputFile, self._pymk)
        self._check_output_file([])

        self.touch('b.out')
        self.assertRaises(Perror.TaskMustHaveOutputFile, self._pymk)
        self._check_output_file([])

    def test_make(self):
        self._template('two_task_dependency_exists1', 'mkfile.py')
        self._import_mkfile()
        self._add_task('/task_6a')

        self._pymk_runtask(['task_6b', 'task_6a'])
        self._pymk_runtask(['task_6b', 'task_6a'])
        self.touch('b.out')
        self._pymk_runtask(['task_6b', 'task_6a'])

    def test_make_2(self):
        self._template('three_task_dependency_exists1', 'mkfile.py')
        self._import_mkfile()
        self._add_task('/task_7a')

        self._pymk_runtask(['task_7c', 'task_7b', 'task_7a'])
        self._pymk_runtask(['task_7c', 'task_7b', 'task_7a'])
        self.touch('b.out')
        self._pymk_runtask(['task_7c', 'task_7b', 'task_7a'])

        self.touch('c.out')
        self._pymk_runtask(['task_7c', 'task_7b', 'task_7a', 'task_7b'])

        self._remove_task('/task_7a')
        self._add_task('/task_7b')
        self._pymk_runtask(['task_7c', 'task_7b', 'task_7a', 'task_7b'])

        self._remove_task('/task_7b')
        self._add_task('/task_7a')
        self._pymk_runtask(['task_7c', 'task_7b', 'task_7a', 'task_7b'])

    def test_uptodate(self):
        self._template('two_task_dependency_exists1', 'mkfile.py')
        self._import_mkfile()
        self._add_task('/task_6a')
        self.touch('b.out', None)

        self._pymk_runtask(['task_6a'])
        self._pymk_runtask(['task_6a'])
        self._pymk_runtask(['task_6a'])


class TaskDependencyFileChangedTest(PymkTestCase):

    def test_make_once(self):
        self._template('two_task_dependency_changed1', 'mkfile.py')
        self._import_mkfile()
        self._add_task('/task_8a')

        self._pymk_runtask(['task_8b', 'task_8a'])
        self._pymk_runtask(['task_8b', 'task_8a'])
        self._pymk_runtask(['task_8b', 'task_8a'])

    def test_make_twice(self):
        self._template('two_task_dependency_changed1', 'mkfile.py')
        self._import_mkfile()
        self._add_task('/task_8a')

        self._pymk_runtask(['task_8b', 'task_8a'])
        self.touch('b.out')
        self._pymk_runtask(['task_8b', 'task_8a', 'task_8a'])
        self._pymk_runtask(['task_8b', 'task_8a', 'task_8a'])

    def test_make_when_depedency_lost(self):
        self._template('two_task_dependency_changed1', 'mkfile.py')
        self._import_mkfile()
        self._add_task('/task_8a')

        self._pymk_runtask(['task_8b', 'task_8a'])
        os.unlink('b.out')
        self._pymk_runtask(['task_8b', 'task_8a', 'task_8b', 'task_8a'])
        self._pymk_runtask(['task_8b', 'task_8a', 'task_8b', 'task_8a'])

    def test_make_three_tasks(self):
        self._template('three_task_dependency_changed1', 'mkfile.py')
        self._import_mkfile()
        self._add_task('/task_12a')

        self._pymk_runtask(['task_12c', 'task_12b', 'task_12a'])
        self._pymk_runtask(['task_12c', 'task_12b', 'task_12a'])

        self.touch('b.out')
        self._pymk_runtask(['task_12c', 'task_12b', 'task_12a', 'task_12a'])
        self._pymk_runtask(['task_12c', 'task_12b', 'task_12a', 'task_12a'])

        self.touch('c.out')
        self._pymk_runtask(['task_12c', 'task_12b', 'task_12a',
                           'task_12a', 'task_12b', 'task_12a'])
        self._pymk_runtask(['task_12c', 'task_12b', 'task_12a',
                           'task_12a', 'task_12b', 'task_12a'])

        os.unlink('b.out')
        self._pymk_runtask(['task_12c', 'task_12b', 'task_12a', 'task_12a',
                           'task_12b', 'task_12a', 'task_12b', 'task_12a'])
        self._pymk_runtask(['task_12c', 'task_12b', 'task_12a', 'task_12a',
                           'task_12b', 'task_12a', 'task_12b', 'task_12a'])

    def test_make_three_tasks_fail1(self):
        self._template('three_task_dependency_changed2', 'mkfile.py')
        self._import_mkfile()
        self._add_task('/task_13a')

        self.assertRaises(Perror.TaskMustHaveOutputFile, self._pymk)
        self._check_output_file(['task_13c', 'task_13b'])

        self.touch('b.out')
        self.assertRaises(Perror.TaskMustHaveOutputFile, self._pymk)
        self._check_output_file(['task_13c', 'task_13b'])

        self.touch('c.out')
        self.assertRaises(Perror.TaskMustHaveOutputFile, self._pymk)
        self._check_output_file(['task_13c', 'task_13b', 'task_13b'])

        os.unlink('b.out')
        self.assertRaises(Perror.TaskMustHaveOutputFile, self._pymk)
        self._check_output_file(
            ['task_13c', 'task_13b', 'task_13b', 'task_13b'])

    def test_make_three_tasks_fail2(self):
        self._template('three_task_dependency_changed3', 'mkfile.py')
        self._import_mkfile()
        self._add_task('/task_14a')

        self.assertRaises(Perror.TaskMustHaveOutputFile, self._pymk)
        self._check_output_file(['task_14c'])

        self.touch('b.out')
        self.assertRaises(Perror.TaskMustHaveOutputFile, self._pymk)
        self._check_output_file(['task_14c'])

        self.touch('c.out')
        self.assertRaises(Perror.TaskMustHaveOutputFile, self._pymk)
        self._check_output_file(['task_14c'])

        os.unlink('b.out')
        self.assertRaises(Perror.TaskMustHaveOutputFile, self._pymk)
        self._check_output_file(['task_14c'])

    def test_make_three_tasks_fail3(self):
        self._template('three_task_dependency_changed4', 'mkfile.py')
        self._import_mkfile()
        self._add_task('/task_15a')

        self.assertRaises(Perror.TaskMustHaveOutputFile, self._pymk)
        self._check_output_file(['task_15c'])

        self.touch('b.out')
        self.assertRaises(Perror.TaskMustHaveOutputFile, self._pymk)
        self._check_output_file(['task_15c', 'task_15c'])

        self.touch('c.out')
        self.assertRaises(Perror.TaskMustHaveOutputFile, self._pymk)
        self._check_output_file(['task_15c', 'task_15c', 'task_15c'])

        os.unlink('b.out')
        self.assertRaises(Perror.TaskMustHaveOutputFile, self._pymk)
        self._check_output_file(
            ['task_15c', 'task_15c', 'task_15c', 'task_15c'])

    def test_uptodate(self):
        self._template('two_task_dependency_changed1', 'mkfile.py')
        self._import_mkfile()
        self._add_task('/task_8a')
        self.touch('b.out', None)

        self._pymk_runtask(['task_8a'])
        self._pymk_runtask(['task_8a'])
        self._pymk_runtask(['task_8a'])


class TaskDependencyLinkTest(PymkTestCase):

    def test_link(self):
        self._template('link_task_1', 'mkfile.py')
        self._import_mkfile()
        self._add_task('/task_linkb')
        self.touch('a.dep.txt', None)
        self.touch('b.dep.txt', None)

        self._pymk_runtask(['task_linka', 'task_linkb'])
        self._pymk_runtask(['task_linka', 'task_linkb'])

        self.touch('a.dep.txt')
        self._pymk_runtask(['task_linka', 'task_linkb', 'task_linka'])

        self.touch('b.dep.txt')
        self._pymk_runtask(
            ['task_linka', 'task_linkb', 'task_linka', 'task_linkb'])


class TaskNameTest(PymkTestCase):

    def test_simple(self):
        _name = 'something'

        class MyTask(Task):
            name = _name
            dependencys = []

        self.assertEqual(_name, MyTask().getName())

    def test_script(self):
        self._template('task_name_1', 'mkfile.py')

        self._import_mkfile()
        self._add_task('/something/usful')
        self._pymk_runtask(['task_namea'])

    def test_arguments(self):
        from json import load
        self._template('task_args_1', 'mkfile.py')

        self._import_mkfile()

        self._set_task('/taska?arg=10', ['Task A'])
        data = load(open('ta.out'))
        self.assertEqual({u'arg': [u'10']}, data)

        self._set_task('/taska?arg=10&arg=12', ['Task A', 'Task A'])
        data = load(open('ta.out'))
        self.assertEqual({u'arg': [u'10', u'12']}, data)

        self._set_task('/taska?arg=10&arg2=12', ['Task A', 'Task A', 'Task A'])
        data = load(open('ta.out'))
        self.assertEqual({u'arg': [u'10'], u'arg2': [u'12']}, data)

    def test_arguments_2_tasks(self):
        from json import load
        self._template('task_args_2', 'mkfile.py')

        self._import_mkfile()

        self._add_task('/taskc?arg=1')
        self._add_task('/taskb?arg2=2')
        self._pymk_runtask(['Task A', 'Task B', 'Task C'])

        data = load(open('ta.out'))
        self.assertEqual({}, data)

        data = load(open('tb.out'))
        self.assertEqual({u'arg2': [u'2']}, data)

        data = load(open('tc.out'))
        self.assertEqual({u'arg': [u'1']}, data)


class GetPathTest(PymkTestCase):

    def test_no_path(self):
        task = TaskTestExample()
        self.assertEqual('/tasktestexample', task.getPath())

    def test_path(self):
        task = TaskTestExample2()
        self.assertEqual('/something', task.getPath())



from mock import patch, MagicMock

from pymk import compare_version
from pymk.tests.base import PymkTestCase
from pymk.script import init_recipe
from pymk.error import CommandError, BadTaskPath
from pymk.task import TaskType, _GetTask


class _GetTaskTest(PymkTestCase):

    def test_parse_task_name(self):
        name, args = _GetTask('/something/elo').parse_url()
        self.assertEqual('/something/elo', name)
        self.assertEqual({}, args)

        name, args = _GetTask('/something/elo?elo=10').parse_url()
        self.assertEqual('/something/elo', name)
        self.assertEqual({'elo': ['10']}, args)

        name, args = _GetTask('/something/elo/?elo=10&zbychu=12').parse_url()
        self.assertEqual('/something/elo/', name)
        self.assertEqual({'elo': ['10'], 'zbychu': ['12']}, args)

        name, args = _GetTask(
            '/something/elo/?elo=10&zbychu=12,10&zbychu=ccc').parse_url()
        self.assertEqual('/something/elo/', name)
        self.assertEqual({'elo': ['10'], 'zbychu': ['12,10', 'ccc']}, args)

    @patch.object(TaskType, 'tasks', {})
    def test_check_if_task_name_exists_fail(self):
        data = _GetTask('/something/elo')
        self.assertRaises(
            BadTaskPath, data.check_if_task_name_exists, '/something/elo')

    @patch.object(TaskType, 'tasks', {})
    def test_check_if_task_name_exists_success(self):
        TaskType.tasks['/something/elo'] = 1
        data = _GetTask('/something/elo')
        result = data.check_if_task_name_exists('/something/elo')
        self.assertEqual(None, result)

    @patch.object(TaskType, 'tasks', {})
    def test_get_task(self):
        TaskType.tasks['/something/elo'] = 2
        data = _GetTask('/something/elo')
        self.assertEqual(2, data.get_task('/something/elo'))

    @patch.object(TaskType, 'tasks', {})
    def test_call(self):
        mock = MagicMock()
        TaskType.tasks['/something/elo'] = mock
        data = _GetTask('/something/elo?elo=10')
        task = data()
        self.assertEqual(mock, task)
        self.assertEqual({'elo': ['10']}, mock._args)


class GraphTest(PymkTestCase):

    def test_error_in_task(self):
        taskname = '/taska'
        self._template('task_graph_error', 'mkfile.py')
        self._import_mkfile()
        self._add_task(taskname)

        self.assertRaises(CommandError, self._pymk_runtask, [])
        task = TaskType.tasks[taskname]
        self.assertTrue(task._error)
        self.assertEqual(
            'shape=circle, regular=1,style=filled,fillcolor=red', task.get_graph_details())


class InitRecipe(PymkTestCase):

    @patch('pymk.script.RecipeType')
    def test_init_recipe(self, RecipeType):
        self.assertTrue(init_recipe('something'))
        RecipeType.getRecipeForModule.assert_called_once_with('something')
        value = RecipeType.getRecipeForModule.return_value
        value.assert_called_once_with()

    @patch('pymk.script.RecipeType')
    def test_init_recipe_with_no_recipe(self, RecipeType):
        RecipeType.getRecipeForModule.return_value = None
        self.assertFalse(init_recipe('something'))
        RecipeType.getRecipeForModule.assert_called_once_with('something')


class CompareVersion(PymkTestCase):

    def test_greater(self):
        self.assertEqual(-1, compare_version('0.3.9', '0.4.0'))
        self.assertEqual(-1, compare_version('0.3.9.1', '0.4.0'))
        self.assertEqual(-1, compare_version('0.3.9.2-custome', '0.4.0'))
        self.assertEqual(-1, compare_version('0.3.9.3.custome', '0.4.0'))
        self.assertEqual(-1, compare_version('0.4.0', '0.4.0.custome'))
        self.assertEqual(-1, compare_version('0.4.0', '0.4.0-custome'))

    def test_equal(self):
        self.assertEqual(0, compare_version('0.4.0', '0.4.0'))

    def test_lower(self):
        self.assertEqual(1, compare_version('0.4.0', '0.3.9'))
        self.assertEqual(1, compare_version('0.4.0', '0.3.9.1'))
        self.assertEqual(1, compare_version('0.4.0', '0.3.9.2-custome'))
        self.assertEqual(1, compare_version('0.4.0', '0.3.9.3.custome'))
        self.assertEqual(1, compare_version('0.4.0.custome', '0.4.0', ))
        self.assertEqual(1, compare_version('0.4.0-custome', '0.4.0'))



import re

VERSION = '0.4.0'


def compare_version(a, b, separator='.', ignorecase=True):
    def _preprocess(v, separator, ignorecase):
        if ignorecase:
            v = v.lower()
        return [int(x) if x.isdigit() else [int(y) if y.isdigit() else y for y in re.findall("\d+|[a-zA-Z]+", x)] for x in v.split(separator)]
    a = _preprocess(a, separator, ignorecase)
    b = _preprocess(b, separator, ignorecase)
    return (a > b) - (a < b)



import os

from pymk import error


class Dependency(object):

    """Base of all dependencys."""
    detailed = []

    def __init__(self):
        self.runned = False

    def __call__(self, task, dependency_force=False):
        """__call__(self, task, dependency_force) -> bool
        Method that will be called to check if dependency need to be rebuilded (if
        it is a task), and return True if task assigned will have to rebuild.
        """
        self.runned = self.do_test(task, dependency_force)
        return self.runned

    # === graph specyfic ===
    def extra(self):
        return ''

    def get_graph_name(self):
        return '"' + str(self.name) + '"'

    def get_graph_details(self):
        return ''

    def _get_shape_color(self):
        if self.runned:
            return 'darkgreen'
        else:
            return 'white'

    def _get_text_color(self):
        return 'black'

    def write_graph_detailed(self, datalog):
        if not self.name in self.detailed:
            self.detailed.append(self.name)
            datalog.write('"%s" [fillcolor=%s,fontcolor=%s,%s];\n' % (
                self.name,
                self._get_shape_color(),
                self._get_text_color(),
                self.get_graph_details())
            )


class FileDependency(Dependency):

    def __init__(self, filenames):
        def convert_filenames(filenames):
            if type(filenames) in [str, unicode]:
                return [filenames]
            else:
                return filenames
        super(FileDependency, self).__init__()
        self.filenames = convert_filenames(filenames)

    @property
    def name(self):
        if len(self.filenames) == 1:
            return self.filenames[0]
        else:
            return 'Many files'

    def compare_mtime(self, first, second):
        if os.path.getmtime(first) > os.path.getmtime(second):
            return True
        else:
            return False


class FileChanged(FileDependency):

    """Dependency returns true if file provided was changed. If task argument is
    provided, then run that task if it should be done."""

    def __init__(self, filenames, task=None):
        super(FileChanged, self).__init__(filenames)
        self.task = task

    def make_dependent_file(self):
        if self.task is None:
            filenames = ', '.join(self.filenames)
            raise error.CouldNotCreateFile(filenames)
        else:
            self.task.run(False)
        return True

    def check_dependent_file(self, task):
        try:
            result = False
            for filename in self.filenames:
                result |= self.compare_mtime(filename, task().output_file)
            return result
        except OSError:
            return self.make_dependent_file()

    def do_test(self, task, dependency_force=False):
        def raise_error_if_task_has_no_output_file(task):
            if task is None:
                return
            if task().output_file is None:
                raise error.TaskMustHaveOutputFile(task.getName())
        #----------------------------------------------------------------------
        raise_error_if_task_has_no_output_file(task)
        raise_error_if_task_has_no_output_file(self.task)

        if dependency_force:
            return True

        if os.path.exists(task().output_file):
            return self.check_dependent_file(task)
        else:
            for filename in self.filenames:
                if not os.path.exists(filename):
                    raise error.CouldNotCreateFile(filename)
            return True

    # === graph specyfic ===
    def extra(self):
        if self.runned:
            return '[label="C",color="blue"]'
        else:
            return '[label="C"]'

    def get_graph_details(self):
        return 'shape=hexagon,regular=1,style=filled,label="%s"' % (
            self.name.replace('/', '\\n')
        )


class FileDoesNotExists(FileDependency):

    """Dependency returns ture if file does not exists."""

    def do_test(self, task, dependency_force=False):
        if dependency_force:
            return True

        for filename in self.filenames:
            if not os.path.exists(filename):
                return True
        return False

    # === graph specyfic ===
    def extra(self):
        if self.runned:
            return '[label="NE",color="blue"]'
        else:
            return '[label="NE"]'

    def get_graph_details(self):
        return 'shape=triangle, regular=1,style=filled,label="%s"' % (
            self.name.replace('/', '\\n')
        )


class AlwaysRebuild(Dependency):

    """Dependency will always make a task rebuild."""

    def __init__(self):
        super(AlwaysRebuild, self).__init__()

    def do_test(self, task, dependency_force=False):
        return True

    # === graph specyfic ===
    def extra(self):
        if self.runned:
            return '[label="A",color="blue"]'
        else:
            return '[label="A"]'

    @property
    def name(self):
        return id(self)

    def get_graph_details(self):
        return 'shape=diamond, regular=1,style=filled,fillcolor=blue,fontcolor=yellow,label="Always\\nRebuild"'


class InnerDependency(Dependency):

    def __init__(self, parent):
        super(InnerDependency, self).__init__()
        self.parent = parent

    # === graph specyfic ===
    @property
    def name(self):
        return self.parent.getName()


class InnerFileExists(InnerDependency):

    def do_test(self, task, dependency_force=False):
        if dependency_force:
            self.parent.run(True, True, True, task)
            return True
        if self.parent().output_file:
            if os.path.exists(self.parent().output_file):
                self.parent.run(False, parent=task)
                return False
            else:
                self.parent.run(parent=task)
                return True
        else:
            raise error.TaskMustHaveOutputFile(self.parent.getName())

    # === graph specyfic ===
    def extra(self):
        if self.runned:
            return '[label="E",color="blue"]'
        else:
            return '[label="E"]'


class InnerFileChanged(InnerDependency):

    def do_test(self, task, dependency_force=False):
        if dependency_force:
            self.parent.run(True, True, True, parent=task)
        ret = self.parent.run(False, parent=task)
        return FileChanged(self.parent().output_file, self.parent)(task) or ret

    # === graph specyfic ===
    def extra(self):
        if self.runned:
            return '[label="C",color="blue"]'
        else:
            return '[label="C"]'


class InnerLink(InnerDependency):

    def do_test(self, task, dependency_force=False):
        self.parent.run(False, parent=task)
        return False

    def extra(self):
        return '[label="L"]'



# -*- encoding: utf-8 -*-
from setuptools import setup, find_packages
from pymk import VERSION

install_requires = [
    'jinja2',
    'argparse',
    'mock',
    'smallsettings',
]

dependency_links = [
]

if __name__ == '__main__':
    setup(name='Pymk',
          version=VERSION,
          description="Make like program, which reads python script as makefile.",
          author='Dominik "Socek" Długajczyk',
          author_email='msocek@gmail.com',
          packages=find_packages(),
          install_requires=install_requires,
          dependency_links=dependency_links,
          test_suite='pymk.tests.get_all_test_suite',
          package_data={'Pymk': [
                        'pymk/tests/tmpl/*.tpl',
                        ]},

          entry_points="""\
            [console_scripts]
                pymk = pymk.script:run
""",
          )



import unittest
import logging
from . import task
from . import dependency
from . import extra
from . import error
from . import script
from . import recipe

all_test_cases = [
    task.TaskTypeTest,
    task.TaskTest,
    task.TaskDependencyFileExistsTest,
    task.TaskDependencyFileChangedTest,
    task.TaskForcing,
    task.TaskDependencyLinkTest,
    task.TaskNameTest,
    task.GetPathTest,

    dependency.FileDoesNotExistsDependencyTest,
    dependency.FileChangedDependencyTest,
    dependency.AlwaysRebuildDependencyTest,
    dependency.InnerFileExistsTest,
    dependency.InnerFileChangedTest,
    dependency.FileDependencyTest,
    dependency.InnerLinkTest,

    extra.TouchTest,
    extra.FindFilesTest,
    extra.RunCmdTest,
    extra.SignalHandlingTest,
    extra.ProccessTest,
    extra.TemplateTest,

    error.ErrorsTests,

    script._GetTaskTest,
    script.GraphTest,
    script.InitRecipe,
    script.CompareVersion,

    recipe.RecipeTest,
    recipe.RecipeTypeTest,
    recipe.DownloadRecipeTest,
]


def get_all_test_suite():
    logging.basicConfig(level=logging.INFO, format="%(asctime)-15s:%(message)s", filename='test.log')
    logging.getLogger('pymk').info('\n\t*** TESTING STARTED ***')
    suite = unittest.TestLoader()
    prepered_all_test_cases = []
    for test_case in all_test_cases:
        prepered_all_test_cases.append(
            suite.loadTestsFromTestCase(test_case)
        )
    return unittest.TestSuite(prepered_all_test_cases)
